import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Control Screen UI at the start of the game.
 */
public class Controls extends Actor
{
  //
  private GreenfootImage image;
  private static final String heading = "Controls";
  private static final String[] controlList = {
   "Click on tower buttons to select a tower to build.",
   "Click on the grid to build the selected tower.",
   "Press the spacebar to spawn a wave of enemies.",
   "Click on built towers to access the upgrade menu.",
   "TIP: Enemies get tougher to kill each round."
 };

  private int width;
  private int height;


  // control screen takes in a width and height
  public Controls(int width, int height){
    this.width = width;
    this.height = height;
    image = new GreenfootImage(width, height);
    setImage(image);
    draw();
  }

  private void draw(){
    int ySpacing = 200;
    // draws the black background
    image.setColor(Color.BLACK);
    image.fill();
    // draws the header and adds it in the middle top of the screen.
    GreenfootImage headerImage = new GreenfootImage(heading, 36, Color.WHITE, Color.BLACK);
    image.drawImage(headerImage, (image.getWidth() / 2) - (headerImage.getWidth()/2) , 100);

    // goes through the array and draws each sentence.
    for(int i = 0; i < controlList.length; i++){
      GreenfootImage text = new GreenfootImage(controlList[i], 18, Color.WHITE, Color.BLACK);
      image.drawImage(text, (image.getWidth()/2) - (text.getWidth()/2) , ySpacing);
      // change the y spacing so the text won't overlap
      ySpacing +=50;
    }
  }

  public void act(){
    // if clicked it will remove the control screen and start the game.
    if (Greenfoot.mouseClicked(this)){
      ((MyWorld) getWorld()).startGame();
      getWorld().removeObject(this);
    }
  }
}
