import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;
/**
 * Class that defines a generic Enemy
 * Class is abstract, can only construct specific types of enemies.
 */
public abstract class Enemy extends Actor
{
// protected variables, these are only accesible to the children of this class.
protected int hp;
protected int score;
protected int movementSpeed;
protected int coinValue;
protected int playerLivesLost;
protected Coord currentTarget;
protected int id;
protected GreenfootSound deathSound;

// a path is an ArrayList of coordinates.
private ArrayList<Coord> path;

    public Enemy(GreenfootImage sprite, ArrayList<Coord> path, int id, int hp, int scoreValue, int speed, int coinValue, int playerLivesLost){
      this.path = copyPath(path);
      this.hp = hp;
      this.score = scoreValue;
      this.movementSpeed = speed;
      this.coinValue = coinValue;
      this.playerLivesLost = playerLivesLost;
      this.id = id;

      //remove the first index, because that would be the spawn point.
      this.path.remove(0);
      //assigned current eneemy target to the first checkpoint, and removes it from
      // the array list.
      this.currentTarget = this.path.remove(0);
      setImage(sprite);
      //builds a new sound object for enemy death.
      deathSound = new GreenfootSound("sounds/Pop_Sound.mp3");
    }

    public void act()
    {
      //PATHFINDING
      // if not at checkpoint it will continue to home onto the next checkpoint.
      if(!atCheckpoint()){
        turnTowards(currentTarget.getX(), currentTarget.getY());
        move(this.movementSpeed);
      } else{
        // if at the checkpoint and the path is not empty ti will pop the next
        // checkpoint from the arraylist and assign it to the current target.
        if(!this.path.isEmpty()){
          this.currentTarget = this.path.remove(0);
        } else {
          // enemy has reached the end of the path, call the event in MyWorld.
           MyWorld m = (MyWorld) getWorld();
           m.enemyGoalReached(this);

        }
      }
    }
    // method to check if an enemy is close enough to the current checkpoint.
    private boolean atCheckpoint(){
      int cx = this.currentTarget.getX();
      int cy = this.currentTarget.getY();

      //buffer to check if at checkpoint
      int BUFFER = 5;
      // within X and Y, it draws a box around the coordinate and checks if
      // enemy is inside that box.
      boolean withinX = (getX() > (cx - BUFFER) && getX() < (cx + BUFFER));
      boolean withinY = (getY() > (cy - BUFFER) && getY() < (cy + BUFFER));
      return withinX && withinY;
    }

    //rebuild path so it avoid refrence issues.
    // method written by my brother.
    private ArrayList<Coord> copyPath(ArrayList<Coord> oldPath){
      ArrayList<Coord> newPath = new ArrayList<Coord>();
      //preforms a Deep copy of of the path passed in
      //since Enemy modifies path directly
      for (Coord p : oldPath){
        newPath.add(new Coord(p.getX(), p.getY()));
      }
      return newPath;
    }

    // method to apply dammage to the enemy
    public void takeDamage(int dmg){
      hp -= dmg;
      // if hp is less than or equal to 0 call event in MyWorld.
      if(hp <= 0){
        deathSound.play();
         ((MyWorld) getWorld()).enemyIsKilled(this);
      }
    }

    // GETTERS

    public int getScore(){
      return this.score;
    }

    public int getCoinValue(){
      return this.coinValue;
    }
    public int getID(){
      return this.id;
    }

    public int getPlayerLivesLost(){
      return this.playerLivesLost;
    }


}
