/**
 * Write a description of class Grid here.
 *
 * @author (your name)
 * @version (a version number or a date)
 */
import greenfoot.*;
import java.util.ArrayList;
public class Grid
{
  private int width;
  private int height;
  private int row;
  private int col;
  private int cellHeight;
  private int cellWidth;

  // 2D array of Tile objects, internal data structure for grid.
  private Tile[][] gameTiles;
  private GridPosition[] currentMap;

  // pre built maps
  public static final GridPosition[] MAP1 = {
    new GridPosition(0,0),
    new GridPosition(0,5),
    new GridPosition(4,5),
    new GridPosition(4,2),
    new GridPosition(9,2)
  };

  public static final GridPosition[] MAP2 = {
    new GridPosition(0,2),
    new GridPosition(6,2),
    new GridPosition(6,6),
    new GridPosition(8,6),
    new GridPosition(8,3),
    new GridPosition(9,3),
  };

  public static final GridPosition[] MAP3 = {
    new GridPosition(0,3),
    new GridPosition(1,3),
    new GridPosition(1,14),
    new GridPosition(4,14),
    new GridPosition(4,4),
    new GridPosition(7,4),
    new GridPosition(7,18),
    new GridPosition(9,18),
    };

    public static final GridPosition[] MAP4 = {
    new GridPosition(0,18),
    new GridPosition(5,18),
    new GridPosition(5,3),
    new GridPosition(7,3),
    new GridPosition(7,19),
    new GridPosition(9,19),

    };
    // constructs the grid of of size widthxHeight
    // requires # rows and columns
    // and requires a map defined above.
    public Grid(int width, int height, int rows, int cols, GridPosition[] map){
      this.width = width;
      this.height = height;
      this.row = rows;
      this.col = cols;
      this.cellHeight = height / row;
      this.cellWidth = width / col;
      this.gameTiles = new Tile[row][col];
      //loads in all the tiles to build the map
      this.currentMap = map;
      loadMap();
    }


    //Creates the Tiles and loads them into 2D array, clears all old data stored
    private void clearMap(){
      int startX = 0;
      int startY = 0;

      //Loops through every spot in the 2D array and builds a Tile object inside
      for (int i = 0; i < row; i++){
        for(int j =0; j < col; j++){
          // get the x and y locations to store inside Tile.
          int currentX = startX + (j * cellWidth);
          int currentY = startY + (i * cellHeight);
          Tile t = new Tile(cellWidth, cellHeight, new Coord(currentX, currentY));
          gameTiles[i][j] = t;
        }
      }
    }

    //loads the image of the grid
    public GreenfootImage getGridImage(){
      GreenfootImage gridImage = new GreenfootImage(width, height);
      for (int i = 0; i < row; i++){
        for (int j = 0; j < col; j++){
          //get the tile object you are currently drawing
          Tile t = gameTiles[i][j];
          //get the x and y coordinates of where that Tile should be
          Coord corner = t.getTopLeftCorner();
          //draw the Tile into that spot
          gridImage.drawImage(t.createImage(), corner.getX(), corner.getY());
        }
      }
      return gridImage;
    }

    //gets the center point of each tile
    public Coord getSnapPoint(Coord mouseCoord){
      GridPosition g = coordToGridPosition(mouseCoord);
      return gameTiles[g.getRow()][g.getCol()].getSnapCoord();
    }

    // getter for the path arrayList.
    public ArrayList<Coord> getPath(){
      ArrayList<Coord> path = new ArrayList<Coord>();
      //itterates through each position on the map variable and converts it
      // to x and y coordinates.
      for (GridPosition gridPos : this.currentMap){
        int row = gridPos.getRow();
        int col = gridPos.getCol();
        //go into the grid, get the tile and its snap coordinates and add to path
        path.add(gameTiles[row][col].getSnapCoord());
      }
      return path;
    }



    //Marks a path on the grid from start row/col to end row/col
    //NOTE make sure that start and end share either the same row /cols
    // method written by my brother.
    private void markPath(GridPosition startPos, GridPosition endPos){

    int startRow = startPos.getRow();
    int startCol = startPos.getCol();

    int endRow = endPos.getRow();
    int endCol = endPos.getCol();

      int r = startRow;
      int c = startCol;
      //until you reach the end row, mark everything along the way
      // Vertical marking
      if (startRow != endRow && startCol == endCol){
        while(r != endRow){
          gameTiles[r][c].setBuildable(false);

          if (r < endRow){
            r++;
          }

          else {
            r--;
          }
        }
      }
      //horizontal marking
      else if (startRow == endRow && startCol != endCol){
        while(c != endCol){
          gameTiles[r][c].setBuildable(false);
          //marking left to right as non-buildable
          if (c < endCol){
            c++;
          }
          //marking right to left as non-buildable
          else {
            c--;
          }
        }
      }
      //to mark the last tile
      gameTiles[r][c].setBuildable(false);


    }
    // getter to check if a tile is buildable
    public boolean isBuildable(Coord pos){
      GridPosition g = coordToGridPosition(pos);

      return gameTiles[g.getRow()][g.getCol()].isBuildable();
    }

    // check if object is outside the grid
    // used for projectiles from towers to delete when off.
    public boolean isOutOfBounds(Coord pos){
      final int BUFFER = 5;
      int x = pos.getX();
      int y = pos.getY();
      boolean outY = y >= (height-BUFFER) || (y < 0 + BUFFER);
      boolean outX = x >= (width-BUFFER) || (x < 0 + BUFFER);

      return outX || outY;

    }

    //setters and getters.

    public int getWidth(){
      return width;
    }

    public int getHeight(){
      return height;
    }


    // sets the tile at the position as either buildable or not.
    public void setOccupied(Coord position, boolean occupied){
      GridPosition g = coordToGridPosition(position);
      gameTiles[g.getRow()][g.getCol()].setOccupied(occupied);
    }

    // helper to load the map.
    private void loadMap(){
      clearMap();
      for(int i =1; i < this.currentMap.length; i++){
        markPath(this.currentMap[i-1], this.currentMap[i]);
      }
    }
    // helper converts xy coordinates to row and column in the grid.
    private GridPosition coordToGridPosition(Coord c){
      int row = c.getY() / cellHeight;
      int col = c.getX() / cellWidth;
      return new GridPosition(row, col);
    }

}
