import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Generic Projectile Class that are shot by towers
 * This class is abstract
 */
public abstract class Projectile extends Actor
{
  private int speed;
  private int damage;
  private Enemy target;

  // constructor for the Projectile.
  public Projectile(GreenfootImage sprite, int speed, int damage, int heading){
    this.speed = speed * 10 ;
    this.damage = damage;
    this.target = target;
    setRotation(heading);
    setImage(sprite);
  }


// checks if projectile is out of the game world
private boolean outOfBounds(){
  // retrieve the grid object from MyWorld
  Grid g = ((MyWorld) getWorld()).getGrid();
  // returns if current position is outside of the grid.
  return g.isOutOfBounds(new Coord(getX(), getY()));
}

  // method that checks if projectile has hit an enemy.
  private void checkHit(){
    Enemy e =(Enemy) getOneIntersectingObject(Enemy.class);
    // if projectile did hit an Enemy
    // call event to take damage in enemy.
    if(e != null){
      e.takeDamage(damage);
      // removes the projctile from the world.
      getWorld().removeObject(this);
    }
  }

// moves the projctile
  public void moveProjectile(){
    // if out of bounds, remove the object from world.
    if(outOfBounds()){
      getWorld().removeObject(this);
    }
    else {
      move(speed);
      checkHit();
    }

  }

  public void act()
  {
    this.moveProjectile();
  }
}
