import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A tile in the grid.
 */

public class Tile
{
  private int width;
  private int height;
  private Coord topLeftCorner;
  private boolean isBuildable;
  private boolean isOccupied;

  // constructor for the tile.
  public Tile(int width, int height, Coord corner){
    this.width = width;
    this.height = height;
    this.topLeftCorner = corner;

    //default value for isBuildable is true;
    this.isBuildable = true;
    // by deafult a tile is not occupied
    this.isOccupied = false;
  }

  // draws the image for the tile
  public GreenfootImage createImage(){
    GreenfootImage tile = new GreenfootImage(width, height);
    // if the tile is buildable it will set it to green.
    if (isBuildable){
      Color tColor = new Color(29, 196, 23);
      tile.setColor(tColor);
    }
    // otherwise set it to gray.
    else {
      Color path = new Color(91, 90, 88);
      tile.setColor(path);
    }
    // draw the tiles with a black border.
    tile.fill();
    tile.setColor(Color.BLACK);
    tile.drawRect(0, 0, width - 1, height - 1);
    return tile;
  }

  // get center coordinate of each tile
  public Coord getSnapCoord(){
    int midX = topLeftCorner.getX() + (width / 2);
    int midY = topLeftCorner.getY() + (height /2);
    return new Coord(midX, midY);
  }

// GETTERS AND SETTERS

  // get the top left corner coordinate of a tile
  public Coord getTopLeftCorner(){
    return topLeftCorner;
  }
  // setter to make each tile as a buildable location
  public void setBuildable(boolean b){
    this.isBuildable = b;
  }

  // setter to mark the tile if it's occupied.
  public void setOccupied(boolean value){
    this.isOccupied = value;
  }

  public boolean isBuildable(){
      return this.isBuildable && (!this.isOccupied);
  }
}
