import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;
/**
 * Generic Tower Class
 * Class is abstract
 */
public abstract class Tower extends Actor
{
  //enumerators, for the different shot tpyes that are available to towers.
  public static enum ShotType {
    BASIC,
    POWER
  }

    private int x;
    private int y;
    private int frame;
    private int fireRate;
    private int range;
    private int targetID;
    private ShotType shotType;
    private int price;
    private int shotLevel;
    private int upgradePrice;



    public Tower(int x , int y, int price ,int fireRate, int range , ShotType s, GreenfootImage sprite){
        this.x = x;
        this.y = y;
        this.fireRate = fireRate;
        this.range = range;
        //makes sure that when you build the tower, it will aquire a target
        this.targetID = -1;
        this.shotType  = s;
        this.price = price;
        this.shotLevel = 1;
        this.upgradePrice = price;
        setImage(sprite);
    }
    // getter for the price of tower
    public int getPrice(){
      return this.price;
    }


    public void act()
    {
      // if clicked on call the event passing in this tower.
      if(Greenfoot.mouseClicked(this)){
        ((MyWorld) getWorld()).setSelectedTowerOnGrid(this);
      }
        // run targeting methods, and shooting methods. and increase frame.
        targeting();
        shootOnInterval();
        frame++;
    }

    // The method was written by my brother.
    private void targeting(){
      List<Enemy> targetList = getObjectsInRange (range, Enemy.class);
      //if you have targets in range
      if (!targetList.isEmpty()) {
        //prefer the target you last tracked
        boolean targetAquired = false;
        for (Enemy e : targetList){
          if (e.getID() == targetID){
            turnTowards(e.getX(), e.getY());
            targetAquired = true;
          }
        }
        //if you could not find your old target, get a new one
        if(!targetAquired) {
          Enemy newTarget = targetList.get(0);
          this.targetID = newTarget.getID();
          turnTowards(newTarget.getX(), newTarget.getY());
        }
      }
    }

    // method for tower to shoot
    private void shootOnInterval(){
      // if the frame matches the fire rate then shoot.
      if(frame % fireRate == 0) {
        //get a list of all objects in range
        List<Enemy> targetList = getObjectsInRange(range, Enemy.class);
        // will only shoot if there is an enemy within the range.
        if (!targetList.isEmpty()){
          Projectile p = determineShot(shotLevel);
          getWorld().addObject(p, this.x , this.y);
        }
      }
    }

    // reads the shot type and spawns the approiate projctile
    private Projectile determineShot(int level){
      // if the shot tpe matches any case spawn that shot.
      switch (this.shotType) {
    		case BASIC:
    			return new BasicProjectile(level, getRotation());
        case POWER:
          return new PowerProjectile(level, getRotation());
    		default:
    			System.out.println("Unsupported Shot, spawning Basic");
    			return new BasicProjectile(level, getRotation());
  		}
    }
    // getter for the tower coordinate
    public Coord getCoord(){
      return new Coord (this.x, this.y);
    }

    //upgrade adds 20% more range and incrases shot level by one.
    public void upgrade(){
      this.range += (range * 0.2);
      this.shotLevel += 1;
      //increase the price to make it even more expensive
      this.upgradePrice+= this.price;
    }

    // GETTERS
    public int getUpgradePrice(){
      return this.upgradePrice;
    }


    public int getShotLevel(){
      return this.shotLevel;
    }
    public int getRange(){
      return this.range;
    }




}
